@@ -9,45 +9,59 @@ use File::Temp qw/ tempfile /;
 use base 'Test::Builder::Module';
 
 our @EXPORT = qw(
-                  exit_is_num
-                  exit_isnt_num
-                  exit_cmp_ok
+   exit_value
+   exit_is_num
+   exit_isnt_num
+   exit_cmp_ok
+   exit_is_defined
+   exit_is_undef
 
-                  stdout_is_eq
-                  stdout_isnt_eq
-                  stdout_is_num
-                  stdout_isnt_num
-                  stdout_like
-                  stdout_unlike
-                  stdout_cmp_ok
-                  stdout_is_file
+   signal_value
+   signal_is_num
+   signal_isnt_num
+   signal_cmp_ok
+   signal_is_defined
+   signal_is_undef
 
-                  stderr_is_eq
-                  stderr_isnt_eq
-                  stderr_is_num
-                  stderr_isnt_num
-                  stderr_like
-                  stderr_unlike
-                  stderr_cmp_ok
-                  stderr_is_file
+   stdout_value
+   stdout_file
+   stdout_is_eq
+   stdout_isnt_eq
+   stdout_is_num
+   stdout_isnt_num
+   stdout_like
+   stdout_unlike
+   stdout_cmp_ok
+   stdout_is_file
 
-                  );
+   stderr_value
+   stderr_file
+   stderr_is_eq
+   stderr_isnt_eq
+   stderr_is_num
+   stderr_isnt_num
+   stderr_like
+   stderr_unlike
+   stderr_cmp_ok
+   stderr_is_file
 
+   );
+                  
 =head1 NAME
 
 Test::Command - Test routines for external commands
 
 =head1 VERSION
 
-Version 0.02
+Version 0.11
 
 =cut
 
-our $VERSION = '0.02';
+our $VERSION = '0.11';
 
 =head1 SYNOPSIS
 
-Test the exit status, STDOUT or STDERR of an external command.
+Test the exit status, signal, STDOUT or STDERR of an external command.
 
    use Test::Command tests => 11;
 
@@ -62,6 +76,12 @@ Test the exit status, STDOUT or STDERR of an external command.
 
    exit_isnt_num($cmd, 0);
 
+   ## testing terminating signal 
+
+   $cmd = 'true';
+
+   signal_is_num($cmd, 0);
+
    ## testing STDOUT
 
    $cmd         = [qw/ echo out /];  ## run as "system @$cmd"
@@ -86,18 +106,28 @@ Test the exit status, STDOUT or STDERR of an external command.
    my $echo_test = Test::Command->new( cmd => 'echo out' );
 
    $echo_test->exit_is_num(0);
+   $echo_test->signal_is_num(0);
    $echo_test->stdout_is_eq("out\n");
 
    ## force a re-run of the command
 
    $echo_test->run;
 
+   ## arbitrary results inspection
+
+   is( $echo_test->exit_value, 0,         'echo exit' );
+   is( $echo_test->signal_value, undef,   'echo signal' );
+   is( $echo_test->stdout_value, "out\n", 'echo stdout' );
+   is( $echo_test->stderr_value, '',      'echo stderr' );
+   is( -s $echo_test->stdout_file, 4,     'echo stdout file size' );
+   is( -s $echo_test->stderr_file, 0,     'echo stderr file size' );
+
 =head1 DESCRIPTION
 
 C<Test::Command> intends to bridge the gap between the well tested functions and
 objects you choose and their usage in your programs. By examining the exit
-status, STDOUT and STDERR of your program you can determine if it is behaving as
-expected.
+status, terminating signal, STDOUT and STDERR of your program you can determine
+if it is behaving as expected.
 
 This includes testing the various combinations and permutations of options and
 arguments as well as the interactions between the various functions and objects
@@ -187,11 +217,12 @@ sub run
    {
    my ($self) = @_;
 
-   my ($exit_status, $stdout_file, $stderr_file) = _run_cmd( $self->{'cmd'} );
+   my $run_info = _run_cmd( $self->{'cmd'} );
 
-   $self->{'result'}{'exit_status'} = $exit_status;
-   $self->{'result'}{'stdout_file'} = $stdout_file;
-   $self->{'result'}{'stderr_file'} = $stderr_file;
+   $self->{'result'}{'exit_status'} = $run_info->{'exit_status'};
+   $self->{'result'}{'term_signal'} = $run_info->{'term_signal'};
+   $self->{'result'}{'stdout_file'} = $run_info->{'stdout_file'};
+   $self->{'result'}{'stderr_file'} = $run_info->{'stderr_file'};
 
    return $self;
 
@@ -311,7 +342,7 @@ sub _build_name
 
    defined $cmd or confess '$cmd is undefined';
 
-   if (ref $cmd eq 'Test::Command')
+   if ( ref $cmd && UNIVERSAL::isa($cmd, 'Test::Command') ) 
       {
       $cmd = $cmd->{'cmd'};
       }
@@ -332,7 +363,7 @@ sub _get_result
 
    defined $cmd or confess '$cmd is undefined';
 
-   if ( ref $cmd eq 'Test::Command' )
+   if ( ref $cmd && UNIVERSAL::isa($cmd, 'Test::Command') ) 
       {
 
       ## run the command if needed
@@ -341,9 +372,7 @@ sub _get_result
          $cmd->run;
          }
 
-      return @{ $cmd->{'result'} }{ qw/ exit_status
-                                        stdout_file
-                                        stderr_file / };
+      return $cmd->{'result'};
       }
    else
       {
@@ -381,7 +410,24 @@ sub _run_cmd
    open STDERR, '>&' . fileno $temp_stderr_fh or confess 'Cannot duplicate temporary STDERR';
 
    ## run the command
-   my $exit_status = system(@{ $cmd }) >> 8;
+   system(@{ $cmd });
+   
+   my $system_return = defined ${^CHILD_ERROR_NATIVE} ? ${^CHILD_ERROR_NATIVE} : $?; 
+   
+   my $exit_status;
+   my $term_signal;
+
+   my $wait_status = $system_return & 127;
+   if ($wait_status)
+      {
+      $exit_status = undef;
+      $term_signal = $wait_status;
+      }
+   else
+      {
+      $exit_status = $system_return >> 8;
+      $term_signal = undef;
+      }
 
    ## close and restore STDOUT and STDERR to original handles
    close STDOUT or confess "failed to close STDOUT: $!";
@@ -389,7 +435,10 @@ sub _run_cmd
    open STDOUT, '>&' . fileno $saved_stdout or confess 'Cannot restore STDOUT';
    open STDERR, '>&' . fileno $saved_stderr or confess 'Cannot restore STDERR';
 
-   return ($exit_status, $temp_stdout_file, $temp_stderr_file);
+   return { exit_status => $exit_status,
+            term_signal => $term_signal,
+            stdout_file => $temp_stdout_file,
+            stderr_file => $temp_stderr_file };
 
    }
 
@@ -398,6 +447,24 @@ sub _run_cmd
 The test routines below compare against the exit status of the executed
 command right shifted by 8 (that is, C<$? E<gt>E<gt> 8>).
 
+=head3 exit_value
+
+   exit_value($cmd)
+
+Return the exit status of the command. Useful for performing arbitrary tests
+not covered by this module.
+
+=cut
+
+sub exit_value
+   {
+   my ($cmd) = @_;
+
+   my $result = _get_result($cmd);
+   
+   return $result->{'exit_status'};
+   }
+
 =head3 exit_is_num
 
    exit_is_num($cmd, $exp_num, $name)
@@ -411,11 +478,11 @@ sub exit_is_num
    {
    my ($cmd, $exp, $name) = @_;
 
-   my ($exit_status) = _get_result($cmd);
-
+   my $result = _get_result($cmd);
+   
    $name = _build_name($name, @_);
 
-   return __PACKAGE__->builder->is_num($exit_status, $exp, $name);
+   return __PACKAGE__->builder->is_num($result->{'exit_status'}, $exp, $name);
    }
 
 =head3 exit_isnt_num
@@ -431,11 +498,11 @@ sub exit_isnt_num
    {
    my ($cmd, $not_exp, $name) = @_;
 
-   my ($exit_status) = _get_result($cmd);
+   my $result = _get_result($cmd);
 
    $name = _build_name($name, @_);
 
-   return __PACKAGE__->builder->isnt_num($exit_status, $not_exp, $name);
+   return __PACKAGE__->builder->isnt_num($result->{'exit_status'}, $not_exp, $name);
    }
 
 =head3 exit_cmp_ok
@@ -452,11 +519,179 @@ sub exit_cmp_ok
    {
    my ($cmd, $op, $exp, $name) = @_;
 
-   my ($exit_status) = _get_result($cmd);
+   my $result = _get_result($cmd);
 
    $name = _build_name($name, @_);
 
-   return __PACKAGE__->builder->cmp_ok($exit_status, $op, $exp, $name);
+   return __PACKAGE__->builder->cmp_ok($result->{'exit_status'}, $op, $exp, $name);
+   }
+
+=head3 exit_is_defined
+
+   exit_is_defined($cmd, $name)
+
+If the exit status of the command is defined, this passes. Otherwise it
+fails. A defined exit status indicates that the command exited normally
+by calling exit() or running off the end of the program.
+
+=cut
+
+sub exit_is_defined
+   {
+   my ($cmd, $op, $exp, $name) = @_;
+
+   my $result = _get_result($cmd);
+
+   $name = _build_name($name, @_);
+
+   return __PACKAGE__->builder->ok(defined $result->{'exit_status'}, $name);
+   }
+
+=head3 exit_is_undef
+
+   exit_is_undef($cmd, $name)
+
+If the exit status of the command is not defined, this passes. Otherwise it
+fails. An undefined exit status indicates that the command likely exited
+due to a signal.
+
+=cut
+
+sub exit_is_undef
+   {
+   my ($cmd, $op, $exp, $name) = @_;
+
+   my $result = _get_result($cmd);
+
+   $name = _build_name($name, @_);
+
+   return __PACKAGE__->builder->ok(! defined $result->{'exit_status'}, $name);
+   }
+
+=head2 Testing Terminating Signal
+
+The test routines below compare against the lower 8 bits of the exit status
+of the executed command.
+
+=head3 signal_value
+
+   signal_value($cmd)
+
+Return the signal code of the command. Useful for performing arbitrary tests
+not covered by this module.
+
+=cut
+
+sub signal_value
+   {
+   my ($cmd) = @_;
+
+   my $result = _get_result($cmd);
+   
+   return $result->{'term_signal'};
+   }
+
+=head3 signal_is_num
+
+   signal_is_num($cmd, $exp_num, $name)
+
+If the terminating signal of the command is numerically equal to the expected number,
+this passes. Otherwise it fails.
+
+=cut
+
+sub signal_is_num
+   {
+   my ($cmd, $exp, $name) = @_;
+
+   my $result = _get_result($cmd);
+   
+   $name = _build_name($name, @_);
+
+   return __PACKAGE__->builder->is_num($result->{'term_signal'}, $exp, $name);
+   }
+
+=head3 signal_isnt_num
+
+   signal_isnt_num($cmd, $unexp_num, $name)
+
+If the terminating signal of the command is B<not> numerically equal to the given
+number, this passes. Otherwise it fails.
+
+=cut
+
+sub signal_isnt_num
+   {
+   my ($cmd, $not_exp, $name) = @_;
+
+   my $result = _get_result($cmd);
+
+   $name = _build_name($name, @_);
+
+   return __PACKAGE__->builder->isnt_num($result->{'term_signal'}, $not_exp, $name);
+   }
+
+=head3 signal_cmp_ok
+
+   signal_cmp_ok($cmd, $op, $operand, $name)
+
+If the terminating signal of the command is compared with the given operand
+using the given operator, and that operation returns true, this passes. Otherwise
+it fails.
+
+=cut
+
+sub signal_cmp_ok
+   {
+   my ($cmd, $op, $exp, $name) = @_;
+
+   my $result = _get_result($cmd);
+
+   $name = _build_name($name, @_);
+
+   return __PACKAGE__->builder->cmp_ok($result->{'term_signal'}, $op, $exp, $name);
+   }
+
+=head3 signal_is_defined
+
+   signal_is_defined($cmd, $name)
+
+If the terminating signal of the command is defined, this passes. Otherwise it
+fails. A defined signal indicates that the command likely exited due to a
+signal.
+
+=cut
+
+sub signal_is_defined
+   {
+   my ($cmd, $op, $exp, $name) = @_;
+
+   my $result = _get_result($cmd);
+
+   $name = _build_name($name, @_);
+
+   return __PACKAGE__->builder->ok(defined $result->{'term_signal'}, $name);
+   }
+
+=head3 signal_is_undef
+
+   signal_is_undef($cmd, $name)
+
+If the terminating signal of the command is not defined, this passes.
+Otherwise it fails. An undefined signal indicates that the command exited
+normally by calling exit() or running off the end of the program.
+
+=cut
+
+sub signal_is_undef
+   {
+   my ($cmd, $name) = @_;
+
+   my $result = _get_result($cmd);
+
+   $name = _build_name($name, @_);
+
+   return __PACKAGE__->builder->ok(! defined $result->{'term_signal'}, $name);
    }
 
 =head2 Testing STDOUT
@@ -464,6 +699,43 @@ sub exit_cmp_ok
 Except where specified, the test routines below treat STDOUT as a single slurped
 string.
 
+=head3 stdout_value
+
+   stdout_value($cmd)
+
+Return the STDOUT of the command. Useful for performing arbitrary tests
+not covered by this module.
+
+=cut
+
+sub stdout_value
+   {
+   my ($cmd) = @_;
+
+   my $result      = _get_result($cmd);
+   my $stdout_text = _slurp($result->{'stdout_file'});
+   
+   return $stdout_text;
+   }
+
+=head3 stdout_file
+
+   stdout_file($cmd)
+
+Return the file name containing the STDOUT of the command. Useful for
+performing arbitrary tests not covered by this module.
+
+=cut
+
+sub stdout_file
+   {
+   my ($cmd) = @_;
+
+   my $result = _get_result($cmd);
+
+   return $result->{'stdout_file'};
+   }
+
 =head3 stdout_is_eq
 
    stdout_is_eq($cmd, $exp_string, $name)
@@ -477,9 +749,9 @@ sub stdout_is_eq
    {
    my ($cmd, $exp, $name) = @_;
 
-   my (undef, $stdout_file) = _get_result($cmd);
+   my $result = _get_result($cmd);
 
-   my $stdout_text = _slurp($stdout_file);
+   my $stdout_text = _slurp($result->{'stdout_file'});
 
    $name = _build_name($name, @_);
 
@@ -499,9 +771,9 @@ sub stdout_isnt_eq
    {
    my ($cmd, $not_exp, $name) = @_;
 
-   my (undef, $stdout_file) = _get_result($cmd);
+   my $result = _get_result($cmd);
 
-   my $stdout_text = _slurp($stdout_file);
+   my $stdout_text = _slurp($result->{'stdout_file'});
 
    $name = _build_name($name, @_);
 
@@ -521,9 +793,9 @@ sub stdout_is_num
    {
    my ($cmd, $exp, $name) = @_;
 
-   my (undef, $stdout_file) = _get_result($cmd);
+   my $result = _get_result($cmd);
 
-   my $stdout_text = _slurp($stdout_file);
+   my $stdout_text = _slurp($result->{'stdout_file'});
 
    $name = _build_name($name, @_);
 
@@ -543,9 +815,9 @@ sub stdout_isnt_num
    {
    my ($cmd, $not_exp, $name) = @_;
 
-   my (undef, $stdout_file) = _get_result($cmd);
+   my $result = _get_result($cmd);
 
-   my $stdout_text = _slurp($stdout_file);
+   my $stdout_text = _slurp($result->{'stdout_file'});
 
    $name = _build_name($name, @_);
 
@@ -565,9 +837,9 @@ sub stdout_like
    {
    my ($cmd, $exp, $name) = @_;
 
-   my (undef, $stdout_file) = _get_result($cmd);
+   my $result = _get_result($cmd);
 
-   my $stdout_text = _slurp($stdout_file);
+   my $stdout_text = _slurp($result->{'stdout_file'});
 
    $name = _build_name($name, @_);
 
@@ -587,9 +859,9 @@ sub stdout_unlike
    {
    my ($cmd, $exp, $name) = @_;
 
-   my (undef, $stdout_file) = _get_result($cmd);
+   my $result = _get_result($cmd);
 
-   my $stdout_text = _slurp($stdout_file);
+   my $stdout_text = _slurp($result->{'stdout_file'});
 
    $name = _build_name($name, @_);
 
@@ -610,9 +882,9 @@ sub stdout_cmp_ok
    {
    my ($cmd, $op, $exp, $name) = @_;
 
-   my (undef, $stdout_file) = _get_result($cmd);
+   my $result = _get_result($cmd);
 
-   my $stdout_text = _slurp($stdout_file);
+   my $stdout_text = _slurp($result->{'stdout_file'});
 
    $name = _build_name($name, @_);
 
@@ -633,10 +905,10 @@ sub stdout_is_file
    {
    my ($cmd, $exp_file, $name) = @_;
 
-   my (undef, $got_file) = _get_result($cmd);
+   my $result = _get_result($cmd);
 
    my ($ok, $diff_start, $got_line, $exp_line, $col_mark) =
-      _compare_files($got_file, $exp_file);
+      _compare_files($result->{'stdout_file'}, $exp_file);
 
    $name = _build_name($name, @_);
 
@@ -661,6 +933,43 @@ EOD
 Except where specified, the test routines below treat STDERR as a single slurped
 string.
 
+=head3 stderr_value
+
+   stderr_value($cmd)
+
+Return the STDERR of the command. Useful for performing arbitrary tests
+not covered by this module.
+
+=cut
+
+sub stderr_value
+   {
+   my ($cmd) = @_;
+
+   my $result      = _get_result($cmd);
+   my $stderr_text = _slurp($result->{'stderr_file'});
+   
+   return $stderr_text;
+   }
+
+=head3 stderr_file
+
+   stderr_file($cmd)
+
+Return the file name containing the STDERR of the command. Useful for
+performing arbitrary tests not covered by this module.
+
+=cut
+
+sub stderr_file
+   {
+   my ($cmd) = @_;
+
+   my $result = _get_result($cmd);
+
+   return $result->{'stderr_file'};
+   }
+
 =head3 stderr_is_eq
 
    stderr_is_eq($cmd, $exp_string, $name)
@@ -674,9 +983,9 @@ sub stderr_is_eq
    {
    my ($cmd, $exp, $name) = @_;
 
-   my (undef, undef, $stderr_file) = _get_result($cmd);
+   my $result = _get_result($cmd);
 
-   my $stderr_text = _slurp($stderr_file);
+   my $stderr_text = _slurp($result->{'stderr_file'});
 
    $name = _build_name($name, @_);
 
@@ -696,9 +1005,9 @@ sub stderr_isnt_eq
    {
    my ($cmd, $not_exp, $name) = @_;
 
-   my (undef, undef, $stderr_file) = _get_result($cmd);
+   my $result = _get_result($cmd);
 
-   my $stderr_text = _slurp($stderr_file);
+   my $stderr_text = _slurp($result->{'stderr_file'});
 
    $name = _build_name($name, @_);
 
@@ -718,9 +1027,9 @@ sub stderr_is_num
    {
    my ($cmd, $exp, $name) = @_;
 
-   my (undef, undef, $stderr_file) = _get_result($cmd);
+   my $result = _get_result($cmd);
 
-   my $stderr_text = _slurp($stderr_file);
+   my $stderr_text = _slurp($result->{'stderr_file'});
 
    $name = _build_name($name, @_);
 
@@ -740,9 +1049,9 @@ sub stderr_isnt_num
    {
    my ($cmd, $not_exp, $name) = @_;
 
-   my (undef, undef, $stderr_file) = _get_result($cmd);
+   my $result = _get_result($cmd);
 
-   my $stderr_text = _slurp($stderr_file);
+   my $stderr_text = _slurp($result->{'stderr_file'});
 
    $name = _build_name($name, @_);
 
@@ -762,9 +1071,9 @@ sub stderr_like
    {
    my ($cmd, $exp, $name) = @_;
 
-   my (undef, undef, $stderr_file) = _get_result($cmd);
+   my $result = _get_result($cmd);
 
-   my $stderr_text = _slurp($stderr_file);
+   my $stderr_text = _slurp($result->{'stderr_file'});
 
    $name = _build_name($name, @_);
 
@@ -784,9 +1093,9 @@ sub stderr_unlike
    {
    my ($cmd, $exp, $name) = @_;
 
-   my (undef, undef, $stderr_file) = _get_result($cmd);
+   my $result = _get_result($cmd);
 
-   my $stderr_text = _slurp($stderr_file);
+   my $stderr_text = _slurp($result->{'stderr_file'});
 
    $name = _build_name($name, @_);
 
@@ -807,9 +1116,9 @@ sub stderr_cmp_ok
    {
    my ($cmd, $op, $exp, $name) = @_;
 
-   my (undef, undef, $stderr_file) = _get_result($cmd);
+   my $result = _get_result($cmd);
 
-   my $stderr_text = _slurp($stderr_file);
+   my $stderr_text = _slurp($result->{'stderr_file'});
 
    $name = _build_name($name, @_);
 
@@ -830,10 +1139,10 @@ sub stderr_is_file
    {
    my ($cmd, $exp_file, $name) = @_;
 
-   my (undef, undef, $got_file) = _get_result($cmd);
+   my $result = _get_result($cmd);
 
    my ($ok, $diff_start, $got_line, $exp_line, $col_mark) =
-      _compare_files($got_file, $exp_file);
+      _compare_files($result->{'stderr_file'}, $exp_file);
 
    $name = _build_name($name, @_);
 
@@ -923,8 +1232,6 @@ under the same terms as Perl itself.
 
 =back
 
-=item * make use of POSIX::W* constants and macros to test exit by signal
-
 =item * potential test functions:
 
 =over 3
